﻿function entityManager() {
    // jsonResultsAdapter
    var jsonResultsAdapter = new breeze.JsonResultsAdapter({

        name: "edmunds",

        extractResults: function (data) {
            var results = data.results;
            if (!results) throw new Error("Unable to resolve 'results' property");
            // Parse only the make and model types
            return results && (results.makeHolder || results.modelHolder);
        },

        visitNode: function (node, parseContext, nodeContext) {
            // Make parser
            if (node.id && node.models) {
                // move 'node.models' links so 'models' can be empty array
                node.modelLinks = node.models;
                node.models = [];
                return { entityType: "Make" }
            }

                // Model parser
            else if (node.id && node.makeId) {
                // move 'node.make' link so 'make' can be null reference
                node.makeLink = node.make;
                node.make = null;

                // flatten styles and sizes as comma-separated strings
                var styles = node.categories && node.categories["Vehicle Style"];
                node.vehicleStyles = styles && styles.join(", ");
                var sizes = node.categories && node.categories["Vehicle Size"];
                node.vehicleSizes = sizes && sizes.join(", ");

                return { entityType: "Model" };
            }
        }
    });

    var DT = breeze.DataType;

    // initializes metadataStore
    function initialize(metadataStore) {
        metadataStore.addEntityType({
            shortName: "Make",
            namespace: "Edmunds",
            dataProperties: {
                id: { dataType: DT.Int64, isPartOfKey: true },
                name: { dataType: DT.String },
                niceName: { dataType: DT.String },
                modelLinks: { dataType: DT.Undefined }
            },
            navigationProperties: {
                models: {
                    entityTypeName: "Model:#Edmunds", isScalar: false,
                    associationName: "Make_Models"
                }
            }
        });

        metadataStore.addEntityType({
            shortName: "Model",
            namespace: "Edmunds",
            dataProperties: {
                id: { dataType: "String", isPartOfKey: true },
                makeId: { dataType: "Int64" },
                makeName: { dataType: "String" },
                makeNiceName: { dataType: "String" },
                name: { dataType: "String" },
                niceName: { dataType: "String" },
                vehicleStyles: { dataType: "String" },
                vehicleSizes: { dataType: "String" },
                categories: { dataType: "Undefined" }
            },
            navigationProperties: {
                make: {
                    entityTypeName: "Make:#Edmunds", isScalar: true,
                    associationName: "Make_Models", foreignKeyNames: ["makeId"]
                }
            }
        });
    };

    // initializes Breeze EntityManager
    var serviceName = "http://api.edmunds.com/v1/api/";

    var ds = new breeze.DataService({
        serviceName: serviceName,
        hasServerMetadata: false,
        useJsonp: true,
        jsonResultsAdapter: jsonResultsAdapter
    });

    var manager = new breeze.EntityManager({ dataService: ds });

    initialize(manager.metadataStore);

    return manager;
};